<?php

namespace Service;

use Exception\CustomException;
use Teknisa\Model\Geral\Entities\Operador;
use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Comments;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\CommentType;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Task;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Teknisa\Model\Teknisa\Entities;

class Call
{
    const TYPE_ISSUE        = Entities\CpCall::TYPE_ISSUE;
    const TYPE_RICH_CLIENT  = Entities\CpCall::TYPE_RICH_CLIENT;
    const TYPE_TICKET  = Entities\CpCall::TYPE_TICKET;


    const MAIN_TYPE_OF_PRODUCT_CALL = "P";
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * @var \Teknisa\Model\Teknisa\Repositories\CpCall
     */
    private $callRepository;
    /**
     * @var Utilities
     */
    private $environment;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager  = $entityManager;
        $this->environment    = InstanceProvider::getEnvironment();
        $this->callRepository = InstanceProvider::getRepository()->getCpCall();
    }

    public function findCalls($clientes,$cdProd, $conditions, $order = array('UPDATED_AT.desc'), $page = 1, $limit = 50): array
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $userId = $this->environment->getCurrentUserId();

        $filterSLA = $filterSLAClas = $filterSLAAss = '';
        //if(!empty($SLA)){
            //$filterSLA = $this->getFilterSLA($SLA);
        //}
        //if(!empty($SLAclas)){
            //list($filterSLAClas,$filterSLAAss) = $this->getFilterSLAclas($SLAclas);
        //}

        $clientePmFilter       = DevOpsUtilities::fixBugWhereINQuery($clientes, "SRE.EXTERNAL_CODE", false, true);
        $clienteIntranetFilter = DevOpsUtilities::fixBugWhereINQuery($clientes, "SER.CDCLIENTE", false, true);
        $clienteCRMFilter      = DevOpsUtilities::fixBugWhereINQuery($clientes, "CLI.CDCLIENTE", false, true);

        $situationFilters = $this->getAllSituationDividedByType();

        $orderBy = '';

        foreach ($order as $item) {
            $orderBy .=' CALL.'. str_replace("."," ",$item) . ',';
        }
        $orderBy = substr($orderBy, 0, -1);

        $customQuery = $this->getQueryAllCall($conditions, $situationFilters, $clientePmFilter,
             $clienteIntranetFilter, $clienteCRMFilter, $filterSLA, $filterSLAClas, $filterSLAAss, $cdProd);

        $result = array();
        if($customQuery) {
            $result = InstanceProvider::getRepository()->getCpCallLocal()->findAllCalls($customQuery,$orderBy, $page, $limit);

            //foreach($result as &$retorno){
                //for($i=1; $i<6;$i++){
                    //$retorno["DATE_" .$i] = $retorno["DATE_". $i] ? $retorno["DATE_". $i] : "";
                //}
            //}
        }
        return $result;
    }

    public function getQueryAllCall($conditions, $situationFilters, $clientePmFilter, $clienteIntranetFilter,
                                    $clienteCRMFilter, $filterSLA = '',$filterSLAClas = '',$filterSLAAss = '', $cdProd = '')
    {
        list($callType, $whereFiltersProductCalls, $situationProductCalls, $specialFiltersProductCalls) = $this->buildWhereFilterToProductCalls($conditions, $situationFilters);
        $mobileProd = $soliProd = [];
        $queryPmCalls = $queryIntranetCalls = $queryCrmCalls = "";

        if(is_array($cdProd)) {
            foreach($cdProd as $cdproduc) {
                if(strlen($cdproduc) !== 3) {
                    array_push($mobileProd, $cdproduc);
                } else {
                    array_push($soliProd, $cdproduc);
                }
            }
        }
        $soliFilter = '';
        if($soliProd) {
            $soliFilter = "AND PRO.CDARVPROD LIKE '%";
            foreach($soliProd as $key => $strProd) {
                if($key > 0) {
                    $soliFilter .= "OR PRO.CDARVPROD LIKE '%";
                }
                $soliFilter .= $strProd . "' ";
            }
        } else {
            $callType = Entities\CpCall::TYPE_ISSUE;
        }
        $mobFilter = '';
        if($mobileProd){
            $mobFilter = "AND PRODUCT IN (";
            foreach($mobileProd as $key => $strProd){
                if($key!=0){
                    $mobFilter .= "," . $strProd;
                }else{
                  $mobFilter .= $strProd  ;
                }
                $mobFilter .= ")";
            }
        } else {
            if($callType) {
                $callType = '';
            } else {
                $callType = Entities\CpCall::TYPE_RICH_CLIENT;
            }
        }
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $cpCallLocalRepository = InstanceProvider::getRepository()->getCpCallLocal();
        if ($callType == Entities\CpCall::TYPE_ISSUE) {
            $queryPmCalls = $cpCallLocalRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter,$filterSLA,$filterSLAClas, $mobFilter);
        } else if ($callType == Entities\CpCall::TYPE_RICH_CLIENT) {
            $queryIntranetCalls = $cpCallLocalRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter, $filterSLA,$filterSLAAss, $soliFilter);
        } else {
            $queryPmCalls = $cpCallLocalRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter, $filterSLA,$filterSLAClas, $mobFilter);
            $queryIntranetCalls = $cpCallLocalRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter,$filterSLA,$filterSLAAss, $soliFilter);
        }

        $customQuery = $queryPmCalls;

        if ($queryIntranetCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryIntranetCalls;
        }

        if ($queryCrmCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryCrmCalls;
        }
        return $customQuery;
    }

    private function buildWhereFilterToProductCalls($conditions, $allSituationDividedByType)
    {
        $specialFilters = '';
        $callType = '';
        $situation = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        foreach($conditions as $key => $filter) {
            if($filter['columnName'] == 'PRODUCT_TYPE') {
                $specialFilters .= " AND EXISTS(SELECT * FROM SOLISERV SOLIC WHERE SOLIC.CDPRODGRUP = ";
                $specialFilters .= "'". $filter['value'] . "' AND SOLIC.NRSOLISERV = C.CALL_RELATIONSHIP)";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_WITHOUT_LABELS_CHECK') {
                if($filter['value'] == '%Y%') {
                    $userId = $this->environment->getCurrentUserId();
                    $specialFilters .= " AND NOT EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN (SELECT NRCALLLABEL FROM CP_CALL_LABEL WHERE CDOPERINCLUSAO = '$userId') OR CLR.LABEL IS NULL)";
                }
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_LABELS') {
                $conditions[$key]["columnName"] = "CLR.LABEL";
                $specialFilters .= "AND EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN(".implode(',',$filter['value'])."))";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'TYPE') {
                $callType = $conditions[$key]['value'];
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'STATUS') {
                $situation = $this->situationFilter($conditions[$key]['value'], $allSituationDividedByType);
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'NRCALL'){
                $conditions[$key]['columnName'] = 'ID';
            } else if($filter['columnName'] == 'CLASSIFICATION_NAME'){
                $conditions[$key]['columnName'] = 'CLASSIFICATION';
            }
        }
        $conditionsQuery = DevOpsUtilities::createQueryFiltered($conditions);

        return array($callType, $conditionsQuery, $situation, $specialFilters);
    }

    public function situationFilter($situationTypes, $allSituationDividedByType, $aliasContato = 'CONT', $aliasTaskStattus = 'TS', $aliasSoliserv = 'SER')
    {
        $situationFilter = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        if(!empty($situationTypes)){
            $issue = array();
            $contato = array();
            $soliserv  = array();
            asort($situationTypes);
            $isWaitingIssue= '';
            foreach ($situationTypes as $situationType) {
                if ($situationType == Entities\CpCall::CLOSE_SITUATION) {
                    $issue = $allSituationDividedByType['PM']['CLOSED'];
                    $contato = $allSituationDividedByType['CRM']['CLOSED'];
                    $soliserv[] = $allSituationDividedByType['GEP']['CLOSED'];
                } else if ($situationType == Entities\CpCall::OPEN_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['OPENED']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['OPENED']);
                    $soliserv[] = $allSituationDividedByType['GEP']['OPENED'];
                } else  if ($situationType == Entities\CpCall::PROGRESS_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['IN_PROGRESS']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['IN_PROGRESS']);
                    $soliserv[] = $allSituationDividedByType['GEP']['IN_PROGRESS'];
                } else if ($situationType == Entities\CpCall::WAITING_SPONSOR_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['WAITING']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['WAITING']);
                    $soliserv[] = $allSituationDividedByType['GEP']['WAITING'];
                    $isWaitingIssue = true;
                }
            }
            $question = <<<SQL
                         'Q' = (SELECT NVL(CONTROL,'I') FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                LEFT JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 )
SQL;

            if(!empty($issue)){
                $contatoString = implode($contato, ',');
                $issueString = implode($issue, ',');
                $soliservString = implode($soliserv, 'OR');
                $situationFilter['CONTATO'] = " AND $aliasContato.NRSEQSITUACAO IN($contatoString)";
                $situationFilter['SOLISERV'] = " AND ($soliservString)";

                if($isWaitingIssue){
                    $situationFilter['ISSUE'] = " AND ($aliasTaskStattus.ID IN($issueString) OR $question )";
                }else{
                    $situationFilter['SOLISERV'] .= "AND NOT ".$allSituationDividedByType['GEP']['WAITING'];
                    $situationFilter['ISSUE'] = " AND $aliasTaskStattus.ID IN($issueString)"." AND NOT ".$question;
                }
            }
        }
        return $situationFilter;
    }

    public function getAllSituationDividedByType($aliasSoliserv = 'SER')
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $waitingTicket    = Utilities::getWaitingTicket();
        $processingType   = Utilities::getExcludedWaitingCustomerLogFaseTram();
        $allSituation     = $cpCallRepository->getAllSituation($this->environment->getCurrentOrganizationId(), $waitingTicket);
        //$cdcliente        = $this->environment->getCurrentCdCliente();
        $cdcliente = 'J02905110000128';
        $waitingGEP       = " (EXISTS (SELECT CDOPERADOR FROM OPERADOR WHERE CDCLIENTE = '$cdcliente' AND CDOPERADOR = $aliasSoliserv.CDOPERRESP AND $aliasSoliserv.CDFASETRAM NOT IN ('$processingType')" ;
        list($openedGep, $closedGep) = $this->getBeginAndEndCdfasetram();
        $allSituationDividedByType = array (
            'GEP' => [
                'CLOSED' => " $aliasSoliserv.CDFASETRAM = $closedGep OR ($aliasSoliserv.IDSOLIENCE = 'S')",
                'OPENED' => " $aliasSoliserv.CDFASETRAM = $openedGep AND ($aliasSoliserv.IDSOLIENCE <> 'S')",
                'WAITING' => $waitingGEP." AND $aliasSoliserv.CDFASETRAM NOT IN ($closedGep) AND ($aliasSoliserv.IDSOLIENCE <> 'S'))) ",
                'IN_PROGRESS' => " $aliasSoliserv.CDFASETRAM NOT IN ($openedGep,$closedGep) AND ($aliasSoliserv.IDSOLIENCE <> 'S') ",
            ],
            'PM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
            'CRM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
        );
        foreach ($allSituation as $situation) {
            $allSituationDividedByType[$situation['PRODUCT']][$situation['SITUATION']][] = $situation['ID'];
        }
        return $allSituationDividedByType;
    }

    public function getBeginAndEndCdfasetram()
    {
        $paramgeral = InstanceProvider::getRepository()->getCpCall()->getBeginAndEndCdfasetram($this->environment->getCurrentOrganizationId());
        return array($paramgeral['CDFATRAMIWEB'], $paramgeral['CDFATRAMENCE']);
    }

    public function findCallComments($callRelationship, $callType)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->callRepository->findOneBy(array('callRelationship' => $callRelationship));
        $commentsRepository = InstanceProvider::getRepository()->getCommentsLocal();

        //$ticket = empty($productCall) ? null : $productCall->getTicketRelationship();
        $userId = $this->environment->getCurrentUserId();
        $comments = [];

        if($callType == Entities\CpCall::TYPE_ISSUE) {
            $comments = $commentsRepository->getCommentsOfTaskExceptCommentedBy($callRelationship);
            $attachments = InstanceProvider::getRepository()->getAttachmentsLocal()->getTaskAttachmentsExceptUploadedBy($callRelationship);
            foreach ($comments['data'] as &$comment) {
                $comment['ATTACHMENT'] = array();
                foreach ($attachments['data'] as $attachment) {
                    if($attachment['COMMENT_ID'] == $comment['ID']){
                        $comment['ATTACHMENT'][] = $attachment;
                    }
                }
                $comment['NOT_EXIST_FILE'] = !count($comment['ATTACHMENT'] );
            }
        } else if($callType == Entities\CpCall::TYPE_RICH_CLIENT) {
            $situationFilters = $this->getAllSituationDividedByType();
            $productCall = $this->callRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            if(!empty($productCall)){
                $comments = $commentsRepository->getProductCallCommentsFromSolicitacao($callRelationship);
                $attachments = InstanceProvider::getService()->getINTRANETIntegration()->getAttachments(null,
                    $callRelationship);
                foreach ($comments['data']  as &$comment) {
                    $comment['ATTACHMENT'] = array();
                    foreach ($attachments as $attachment) {
                        if($attachment['FOLDER'] === $comment['ORDERBY'] ){
                            $comment['ATTACHMENT'][] = $attachment;
                        }
                    }
                    $comment['NOT_EXIST_FILE'] = !count($comment['ATTACHMENT'] );
                }
            }
        }
        return $comments;
    }

    public function findCallAttachments($callRelationship, $callType, $commentId, $filename)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->callRepository->findOneBy(array('callRelationship' => $callRelationship));
        $callId = empty($productCall) ? null : $productCall->getNrcall();

        if($callType == Entities\CpCall::TYPE_ISSUE) {
            $commentIdQuery = "";
            if($commentId) {
                $commentIdQuery = "AND C.ID = " . $commentId;
            }
            $attachments = InstanceProvider::getRepository()->getAttachmentsLocal()->getTaskAttachmentsExceptUploadedBy($callRelationship, $commentIdQuery);
            foreach($attachments['data'] as $index => $attachment) {
                $attachments[$index]['__fromIssue'] = true;
            }
        } else {
            $result = InstanceProvider::getService()->getINTRANETIntegration()->getAttachments($callId,
                $callRelationship, $filename);
            $attachments = array('data'=> $result,'pagination' => array('limit'=> 999, 'page' => 1, 'total' => count($result)));
        }
        return  $attachments;
    }

    public function getCall( $callRelationship, $callType)
    {
        $call = array();
        if($callType == self::TYPE_RICH_CLIENT || $callType == self::TYPE_ISSUE){
            $call = $this->findProductCall($callRelationship, $callType);
        }

        return $call;
    }

    public function findCallTypeIssue($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCallLocal();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeIssue($id,$situationFilters);
    }

    public function findCallTypeRichClient($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCallLocal();
        $userId = $this->environment->getCurrentUserId();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeRichClient($id,$userId,$situationFilters);
    }

    public function getCallTypeDescription($type) {
        switch($type) {
            case self::TYPE_ISSUE:
                $desc = Entities\CpCall::TYPE_ISSUE_DESC;
                break;
            case self::TYPE_RICH_CLIENT:
                $desc = Entities\CpCall::TYPE_RICH_CLIENT_DESC;
                break;
            default:
                $desc = '';
        }
        return $desc;
    }

    private function findProductCall($callRelationship, $callType)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCallLocal();
        $soliservRepository = InstanceProvider::getRepository()->getSoliservLocal();
        $userId             = $this->environment->getCurrentUserId();
        $productCall        = array();

        $situationFilters = $this->getAllSituationDividedByType();
        /** @var Entities\CpCall $call */
        $call = InstanceProvider::getRepository()->getCpCall()->findOneBy(array('callRelationship' => $callRelationship));
        if(!empty($call)){
            if($call->getType() == \Service\Call::TYPE_ISSUE) {
                $productCall = $this->findCallTypeIssue($call->getNrcall());
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }

            } else if ($call->getType() == \Service\Call::TYPE_RICH_CLIENT) {
                $productCall = $this->findCallTypeRichClient($call->getNrcall());
                $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }
                /** @var Entities\Soliserv $soliServ */
                $soliServ = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));
                $productCall['COMMENT_PUBLIC'] = $this->solservCommentPublic($soliServ);

                $useLicense = $this->getLicencausoBySoliserv($soliServ);
                if($useLicense){
                    $productCall['USE_LICENSE'] = $useLicense->getNrlicencauso();
                    $productCall['USE_LICENSE_LABEL'] = $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso();
                }else{
                    $productCall['USE_LICENSE'] = '';
                    $productCall['USE_LICENSE_LABEL'] = '';
                }
            }
        } else if ($callType == "I"){
            $productCall = $cpCallRepository->findIssueById($callRelationship, $situationFilters);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['ID'] = null;
            $productCall['APPOINTMENT_DATE_INIT'] = null;
            $productCall['APPOINTMENT_DATE'] = null;
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['CALL_LABELS'] = empty($productCall['CALL_LABELS']) ? array() : explode(',', $productCall['CALL_LABELS']);
        } else if ($callType == "S"){
            $productCall = $cpCallRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['ID'] = null;
            $productCall['APPOINTMENT_DATE_INIT'] = null;
            $productCall['APPOINTMENT_DATE'] = null;

            /** @var Entities\Soliserv $soliServ */
            $soliServ = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));
            $productCall['COMMENT_PUBLIC'] = $this->solservCommentPublic($soliServ);

            $useLicense = $this->getLicencausoBySoliserv($soliServ);

            $productCall['USE_LICENSE'] = $useLicense ? $useLicense->getNrlicencauso() : '';
            $productCall['USE_LICENSE_LABEL'] = $useLicense ? $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso() : '';

        }

        if(!empty($productCall)){
            $productCall['MAIN_TYPE_CALL'] = self::MAIN_TYPE_OF_PRODUCT_CALL;
        }

        return $productCall;
    }

    /**
     * @param \Teknisa\Model\Teknisa\Entities\Soliserv  $soliserv
     * @return null |\Teknisa\Model\Teknisa\Entities\Licencauso
     */
    private function getLicencausoBySoliserv(\Teknisa\Model\Teknisa\Entities\Soliserv $soliserv = null)
    {
        $licencauso = null;
        if ($soliserv->getCdsetor() != null) {
            $id = array(
                "cdcliente" => $soliserv->getCdcliente(),
                "nrlicencauso" => $soliserv->getNrlicencauso(),
                "nrorg" => $soliserv->getNrorg(),
            );
            $licencauso = InstanceProvider::getRepository()->getLicencauso()->findOneBy($id);
        }
        return $licencauso;
    }

    public function findProductCallProductsNew($cdCliente,  $callType = null, $callRelationship = null, $cdprodcomp = null)
    {
//        $filter = '';
//        if($callType){
//            $filter = " AND CALL_TYPE = '".$callType."'";
//        }
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCallLocal();
        return $cpCallRepository->findProductCallProductsNew($cdCliente,  $callType, $callRelationship, $cdprodcomp);
    }

    public function findProductCallClassificationFromRichClient()
    {
        return InstanceProvider::getRepository()->getCpCallLocal()->getClassificationSoli();
    }

    public function findProductCallClassificationIssue()
    {
        $nrOrg          = Utilities::getEnvironment()->getCurrentOrganizationId();
        return InstanceProvider::getRepository()->getCpCallLocal()->findProductCallClassificationIssue($nrOrg);
    }

    public function findProductCallProductTypes($productCode)
    {
        return InstanceProvider::getRepository()->getCpCallLocal()->findProductCallProductTypes($productCode);
    }
    public function findProductCalProductVersions($productType, $product, $productCode = null)
    {
        $productVersion = InstanceProvider::getRepository()->getCpCallLocal()->getVersao($productType, $product, null, null, $productCode);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    public function findProductVersionLtsWithoutBuild($pmProjectId, $cdcliente, $nrexternalProductId = null)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource  $resource */
        $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $cdcliente));

        if ($nrexternalProductId) {
            $pmProjectId = InstanceProvider::getRepository()->getCpCall()->getProjectIdByProductDevOpsId($nrexternalProductId);
        }

        return InstanceProvider::getService()->getProductVersion()->findProductVersionLtsWithoutBuild($pmProjectId, $resource->getId());
    }

    public function findProductCallProductReleases($productType, $version)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCallLocal();
        $productVersion = $cpCallRepository->getReleases($productType, $version);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    public function findProductCalProductPatch($productType, $version, $release)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCallLocal();
        $productVersion = $cpCallRepository->getPatch($productType, $version, $release);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    public function findAllOperadorResponsible(){
        return InstanceProvider::getRepository()->getCpCallLocal()->findAllOperadorResponsible();
    }

    public function findAllCustomers(){
        return InstanceProvider::getRepository()->getCpCallLocal()->findAllCustomers();
    }

    public function findProductCallUseLicense($cdCliente){
        return InstanceProvider::getRepository()->getCpCallLocal()->findProductCallUseLicense($cdCliente);
    }

    public function findAllCallLabels() {
        return InstanceProvider::getRepository()->getCpCallLocal()->getAllCallLabels();
    }

    public function findMenuItems($pmProductId,$cdCliente)
    {
        return InstanceProvider::getRepository()->getCpCallLocal()->findMenuItems($pmProductId,$cdCliente);

        /* Utilizar esse formato quando o componente ZdSelectTree estiver funcionando. 
        $data = InstanceProvider::getRepository()->getCpCallLocal()->findMenuItems($pmProductId,$cdCliente);
        $tree = $this->buildTreeFromArray($data['data']);
        $data['data'] = !empty($tree) ? $tree : $data['data'];
        return $data;
        */
    }

    /**
     * @param $items
     * @return array
     */
    public function buildTreeFromArray($items): array
    {
        $children = [];

        foreach ($items as &$item) {
            $children[$item['parent_id'] ?? 0][] = &$item;
        }

        unset($item);

        foreach ($items as &$item) {
            if (isset($children[$item['id']])) {
                $item['children'] = $children[$item['id']];
            }
        }

        return $children[0] ?? [];
    }


    public function findProductVersion($pmProductId)
    {
        $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        return InstanceProvider::getRepository()->getCpCallLocal()->findProductVersionWithConcatVersionByProduct($pmProductId,$organizationId);
    }

    public function saveNewProductCall($callType, $reference, $product, $productType, $productVersion, $useLicense,
                                       $classification, $board, $description, $reportedBy, $callLabels, $productRelease,
                                       $productPatch, $callRelationship = null,$devopsId = null, $menuItemId = null,
                                       $menuVersionId = null,$cdcliente = null, $invisibleFields = null,$appointmentDate = null)
    {
        if(empty($reportedBy)){
            $reportedBy = InstanceProvider::getEnvironment()->getCurrentCdoperador();
        }
        if($callType == \Service\Call::TYPE_ISSUE) {
            $newProductCall = $this->saveNewIssueTypeProductCall($callType, $reference, $product, $classification, $board, $description,
                $reportedBy, $callLabels, $callRelationship,$devopsId,$menuItemId,$menuVersionId,$cdcliente, $invisibleFields,$appointmentDate);
        } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $newProductCall = $this->saveNewRichClientTypeProductCall($callType, $reference, $product, $productType, $productVersion,
                $useLicense, $classification, $description, $reportedBy, $callLabels, $productRelease, $productPatch,
                $callRelationship,$cdcliente,$appointmentDate);
        } else {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_SAVE_CALL_DUE_TO_UNKNOWN_TYPE');
        }
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();

        /** @var Entities\CpCall $cpCall */
        $cpCall = $cpCallRepository->find($newProductCall['ID']);

        $newProductCall['CALL_RELATIONSHIP'] = $cpCall->getCallRelationship();

        return $newProductCall;
    }

    public function findResourceById($resourceId) {
        return InstanceProvider::getRepository()->getResource()->findOneBy(array('id' => $resourceId));
    }

    public function findResourceByExternalCode($externalCode) {
        return InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode'=>$externalCode));
    }

    public function createIssueInPM($reference, $product, $classification, $board, $description,
                                    $reportedBy, $callRelationship = null, $cdCliente = null)
    {
        $issue = null;
        $clienteRepository = InstanceProvider::getRepository()->getCliente();

        /** @var \Teknisa\Model\Geral\Entities\Operador $reporter */
        $reporter = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $reportedBy));
        $reporterName = $reporter->getNmoperador();

//        if (!$cdCliente) {
//            $cdCliente = $this->environment->getCurrentCdCliente();
//            if(!$cdCliente){
//                $this->environment->setCdClienteCurrent($cdCliente);
//            }
//        }

        $client = $clienteRepository->findOneBy(array(
            'cdcliente' => $cdCliente,
            'nrorg' => $reporter->getNrorg()
        ));
        $reporterFrom = empty($client) ? null : $client->getNmfantclie();
        $sponsor = $this->findResourceByExternalCode($cdCliente);
        $clienteResource = $this->findResourceByExternalCode($reporter->getCdcliente());

        if (empty($sponsor)) {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_ISSUE_BECAUSE_CLIENT');
        }
        if (empty($clienteResource)) {
            /** @var \Teknisa\Model\Geral\Entities\Cliente $cliente */
            $cliente = $clienteRepository->findOneBy(array('cdcliente' => $cdCliente));
            $name = $cliente->getNmfantclie();
            throw new CustomException('ERR_CREATE_OR_UPDATE_PRODUCT_CALL_NOT_CUSTOMER_PM', 500, array('customerName' => $name));
        }

        if (!$callRelationship) {
            $issue = InstanceProvider::getService()->getIssue()->createIssueByProductCall($reference, $product, $classification,
                $description, $board, $reporterName, $reporterFrom, $sponsor->getId(), $reportedBy);
            if (empty($issue)) {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_ISSUE_BASED_ON_PRODUCT_CALL');
            }

            $callRelationship = $issue;
        }
        return $callRelationship;
    }

    public function saveNewIssueTypeProductCall($callType, $reference, $product, $classification, $board, $description,
                                                $reportedBy, $callLabels, $callRelationship = null,$devopsId = null,
                                                $menuItemId = null,$menuVersionId = null, $cdcliente = null, $invisibleFields = null,$appointmentDate = null)
    {
        $connection           = $this->entityManager->getConnection();
        $cpCallRepository     = InstanceProvider::getRepository()->getCpCall();
        $issue = null;
        try {
            $connection->beginTransaction();

            if($devopsId){
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $devopsProduct */
                $devopsProduct = InstanceProvider::getRepository()->getProduct()->findOneBy(array('id'=> $devopsId));
                if($devopsProduct && $devopsProduct->getProject()){
                    $board = $devopsProduct->getProject()->getId();
                }
            }

            if($callRelationship){
                $cpCall = $cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
                if (!empty($cpCall)){
                    throw new CustomException('ERR_CALL_ALREADY_RELATED_ISSUE');
                }
                $resource = $this->findResourceById($reportedBy);
                if(!empty($resource) && $resource->getExternalCode()){
                    $reportedBy = $resource->getExternalCode();
                } else {
                    $reportedBy = $this->environment->getCurrentUserId();
                }
            }

            $callRelationship = $this->createIssueInPM($reference, $product, $classification,
                $board, $description, $reportedBy, $callRelationship, $cdcliente);

            $issue = $this->setVersionIssue($callRelationship, $menuItemId, $menuVersionId);

            $productCall = $this->callRepository->findOneBy(array('callRelationship'=> $callRelationship));
            if(!$productCall){
                $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference,
                    $reportedBy, $description,  $callType, $callRelationship, $product, $classification);
                $this->entityManager->persist($productCall);
            }

            if(!empty($appointmentDate)){
                $productCall->setDtconpcuren($appointmentDate);
            }
            $productCall->setProduct($product);
            $this->entityManager->flush();

            $this->setFirstCommentIssue($product, $reportedBy, $callLabels, $menuItemId, $menuVersionId,
                $issue, $productCall);
            if($invisibleFields){
                //$this->updateInvisibleFieldsInCall($productCall->getNrcall(), $invisibleFields);
            }

            $this->entityManager->flush();
            $connection->commit();

            return array("ID" => $productCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    private function solservCommentPublic(Entities\Soliserv $soliServ)
    {
        $return = false;
        /** @var Operador $operesp */
        $operesp = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $soliServ->getCdoperresp()));
        if ($operesp && $operesp->getCdcliente() == Utilities::getCdClienteBase()) {
            $return = true;
        }
        return $return;
    }

    public function getQuestionForSendCall($lang, $callType, $callRelationship = null)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCallLocal();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        if($callType == self::TYPE_RICH_CLIENT){
            /** @var Entities\Soliserv $soliserv */
            $soliserv = $soliservRepository->findOneBy(array( 'nrsoliserv' => $callRelationship));
            $cdfasetram = $soliserv->getCdfasetram();
        }else{
            throw new \Exception('ERROR_CALL_TYPE');
        }
        $return = $cpCallRepository->getInfoForSendCall($cdfasetram, Utilities::langForIdidioma($lang),$nrorg);

        return $return;
    }

    public function operadorByResponse($callType, $cdfastramdes, $nrfastramdes, $callRelationship,$cdCliente)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCallLocal();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();

        if($callType == self::TYPE_ISSUE){
            $return = $cpCallRepository->operadorByResponseIssue($callRelationship);
        }else{
            $return = [];
            if($cdfastramdes && $nrfastramdes){
                /** @var Entities\Soliserv $soliserv */
                $soliserv = $soliservRepository->findOneBy(array( 'nrsoliserv' => $callRelationship));
                list($iddepacaoope,$cdgroperresp) = $cpCallRepository->operadorGrupoByResponse($soliserv->getCdfasetram(),
                    $cdfastramdes, $nrfastramdes);
                $return = $cpCallRepository->operadorByResponse($iddepacaoope,$cdgroperresp,$cdCliente);
            }
        }
        return $return;
    }


    public function sendCallGep($nrcall, $cdfastramdes, $cdoperador, $description, $nrfastramdes,
                                $txacaofatram, $callType = null, $callRelationship = null, $documents = array())
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
                /** @var Entities\Soliserv $soliserv */
                $soliserv = $soliservRepository->findOneBy(array('nrsoliserv' => $callRelationship));
                $nrsoliserv = $callRelationship;
                $paramGeral         = $cpCallRepository->getallParamGeral(20);
                $cdOperadorCurrent  = $this->environment->getCurrentCdoperador();

                $soliserv->setCdfasetram($cdfastramdes);
                if ($soliserv->getCdfasetram() == $paramGeral['CDFATRAMENCE']) {
                    if($soliserv->getCdoperresp() == $cdOperadorCurrent){
                        $soliserv->setIdsolilida('S');
                    }else{
                        $soliserv->setIdsolilida('N');
                    }
                    $soliserv->setIdsolience('S');
                    $soliserv->setIdstatussoli('C');
                    $soliserv->setCdoperresp($cdOperadorCurrent);
                    $now = new \DateTime();
                    $soliserv->setDtencesoli($now);
                }else{
                    if($soliserv->getCdoperresp() == $cdoperador){
                        $soliserv->setIdsolilida('S');
                    }else{
                        $soliserv->setIdsolilida('N');
                    }
                    $soliserv->setCdoperresp($cdoperador);
                }
                $this->entityManager->persist($soliserv);
                $dt             = new \DateTime();
                $dtString = $dt->format(\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $nrorg          = 20;
                $cdfilial       = $paramGeral['CDFILIAL'];
                $cdFaseTram     = $cdfastramdes;
                $cdOperResp     = $soliserv->getCdoperresp();
                $txacaolftram   = $txacaofatram;
                $dsAlFaseTram   = $description;
                list($logFaseTramInsertionQuery, $logFaseTramInsertionQueryParams) = InstanceProvider::getService()->getIntranetIntegration()->
                logFaseTramFactory->createLogFaseTramInsertionQuery($nrorg, $cdfilial, $nrsoliserv, $dtString,
                    $cdFaseTram, $cdOperadorCurrent, $cdOperResp, $txacaolftram, $dsAlFaseTram,$cdOperadorCurrent);
                $connection->executeQuery($logFaseTramInsertionQuery, $logFaseTramInsertionQueryParams);

                $dateString = $dt->format('YmdHis');
                $this->entityManager->flush();

                InstanceProvider::getService()->getIntranetIntegration()->insertFile($documents, $callRelationship, $dateString);


                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function setVersionIssue($callRelationship, $menuItemId = null, $menuVersionId = null)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $issue */
        $issue = InstanceProvider::getRepository()->getTask()->find($callRelationship);
        if ($menuVersionId) {
            $productVersion = InstanceProvider::getRepository()->getProductVersion()->findOneBy(array('id' => $menuVersionId));
            $issue->setProductVersion($productVersion);
        }
        $issue->setMenuItem($menuItemId);
        $this->entityManager->persist($issue);
        return $issue;
    }

    public function setFirstCommentIssue($product, $reportedBy, $callLabels = array(), $menuItemId, $menuVersionId,
                                         $issue, $productCall)
    {
        /** @var Comments $fistCommnet */
        $firstCommentDescription = InstanceProvider::getRepository()->getComments()
            ->getFistCommentsWithMessageVersion($issue->getId(), Messages::ISSUE_OPEN_VERSION, $product);
        if ($firstCommentDescription && ($menuVersionId || $menuItemId)) {
            $fistComment = InstanceProvider::getRepository()->getComments()->find($firstCommentDescription['ID']);
            $fistComment->setDescription($firstCommentDescription['DESCRIPTION']);
            $this->entityManager->persist($fistComment);
        }
        InstanceProvider::getService()->getCallLabels()->createCallRelationWithLabels($productCall->getNrcall(), $callLabels, $reportedBy);
    }

    private function updateInvisibleFieldsInCall($id, $invisibleFields){
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $callFactory      = InstanceProvider::getFactory()->getCall();

        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $cpCallRepository->find($id);

        for($i=1;$i<6;$i++){
            if($invisibleFields["DATE_" . $i]){
                $invisibleFields["DATE_" . $i] = \Util\DateUtil::getDataDeString($invisibleFields["DATE_" . $i],\Util\DateUtil::FORMATO_BRASILEIRO, true);

            }
        }
        $productCall = $callFactory->updateCallAddInvisibleFields($productCall, $invisibleFields);
        $this->entityManager->persist($productCall);

    }
    public function saveNewRichClientTypeProductCall($callType, $reference, $product, $productType, $productVersion,
                                                     $useLicense, $classification, $description, $reportedBy,
                                                     $callLabels, $productRelease, $productPatch, $callRelationship = null,$cdCliente = null,$appointmentDate = null)
    {
        $connection         = $this->entityManager->getConnection();
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $callLabelsService  = InstanceProvider::getService()->getCallLabels();
        try {
            $connection->beginTransaction();
            if(!$cdCliente){
                $cdCliente = Utilities::getCdClienteBase();
            }
            $callRelationship = $this->saveSolicitacao($reference, $product, $productType, $productVersion, $useLicense,
                $classification, $description, $reportedBy, $productRelease, $productPatch,$cdCliente, $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array("callRelationship" => $callRelationship));
            if (!$productCall) {
                $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference, $reportedBy, $description, $callType,
                    $callRelationship, $product, $classification);

                if(!empty($appointmentDate)){
                    $productCall->setDtconpcuren($appointmentDate);
                }
                $this->entityManager->persist($productCall);
                $this->entityManager->flush();

                $callLabelsService->createCallRelationWithLabels($productCall->getNrcall(), $callLabels, $reportedBy);

                $connection->commit();

            }

            return array("ID" => $productCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveSolicitacao($reference, $product, $productType, $productVersion, $useLicense, $classification,
                                    $description, $reportedBy, $productRelease, $productPatch,$cdCliente, $callRelationship = null)
    {
        $operadorRepository = InstanceProvider::getRepository()->getOperador();
        $reporter = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
        if (!$reporter) {
            $reporter = $operadorRepository->findOneBy(array('cdoperador' => '1'));
        }

        if (!$callRelationship) {
            $P_CDCLIENTE = $cdCliente;
            $P_CDOPERADOR = $reporter->getCdoperador();
            $P_DSREFESOLI = $reference;
            $P_CDASSUNTO = $classification;
            $P_CDGRUPSIST = $productType;
            $P_CDSISTEMA = $product;
            $P_NRLICENCAUSO = $useLicense;
            $P_DSSOLISERV = $description;
            $P_DSVERSAO = $productVersion;
            $P_DSRELEASE = $productRelease;
            $P_DSPATCH = $productPatch;

            $callRelationship = InstanceProvider::getService()->getIntranetIntegration()->createAndPersistSolicitacao($P_CDCLIENTE, $P_CDOPERADOR,
                $P_DSREFESOLI, $P_CDASSUNTO, $P_CDGRUPSIST, $P_CDSISTEMA, $P_NRLICENCAUSO, $P_DSSOLISERV, $P_DSVERSAO,
                $P_DSRELEASE, $P_DSPATCH);

        }
        return $callRelationship;
    }

    public function updateProductCall($callType, $id,  $reference, $product, $productType, $productVersion,
                                      $useLicense, $classification, $board, $description, $reportedBy,
                                      $callLabels, $devopsId, $menuItemId, $menuVersionId, $cdCliente,$callRelationship,
                                      $appointmentDate,$productRelease,$productPatch, $versionDevId)
    {
        $id = $this->validadeSaveCall($callType, $id, $reference, $product, $classification, $description, $reportedBy, $callRelationship);
        if($callType == \Service\Call::TYPE_ISSUE) {
            $this->updateIssueTypeProductCall($id, $reference, $product, $classification, $board, $description,
                $reportedBy, $callLabels,$devopsId,$menuItemId,$menuVersionId, $cdCliente,$appointmentDate, $versionDevId);
        } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $this->updateRichClientTypeProductCall($id, $reference, $product, $productType, $productVersion, $useLicense,
                $classification, $board, $description, $reportedBy, $callLabels,$cdCliente, $appointmentDate,$productRelease,$productPatch);
        } else {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_SAVE_CALL_DUE_TO_UNKNOWN_TYPE');
        }
    }

    private function updateIssueTypeProductCall($id, $reference, $product, $classification, $board, $description,
                                                $reportedBy, $callLabels,$devopsId, $menuItemId ,
                                                $menuVersionId ,$cdCliente,$appointmentDate = null, $versionDevId = null)
    {
        $connection = $this->entityManager->getConnection();
        $cpCallRepository       = InstanceProvider::getRepository()->getCpCall();
        $operadorRepository     = InstanceProvider::getRepository()->getOperador();
        $clienteRepository      = InstanceProvider::getRepository()->getCliente();
        $callFactory            = InstanceProvider::getFactory()->getCall();
        $isSameProduct = false;
        try {
            $connection->beginTransaction();
            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            $productCall = $cpCallRepository->find($id);
            if($devopsId){
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $devopsProduct */
                $devopsProduct = InstanceProvider::getRepository()->getProduct()->findOneBy(array('id'=> $devopsId));
                if($devopsProduct && $devopsProduct->getProject()){
                    $board = $devopsProduct->getProject()->getId();
                }
            }

            if($productCall->getProduct() == $product){
                $isSameProduct = true;
            }
            $productCall = $callFactory->updateCall($productCall, $reference, $product, $classification, $description);
            $reporter = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
            $reporterName = $reporter->getNmoperador();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource  $resource */
            $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $reportedBy));
            $resourceSponsor = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $cdCliente));

            $client = $clienteRepository->findOneBy(array(
                'cdcliente' => $cdCliente,
                'nrorg' => $reporter->getNrorg()
            ));

            $reporterFrom = empty($client) ? null : $client->getNmfantclie();

            $issueId = $productCall->getCallRelationship();

            if($isSameProduct){
                $board = null;
            }

            $issue = InstanceProvider::getService()->getIssue()->updateIssueByProductCall($issueId, $reference, $product,
                                                    $classification, $description, $board, $resource, $resourceSponsor->getId());

            if (empty($issue)) {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_UPDATE_ISSUE_BASED_ON_PRODUCT_CALL');
            }

            /** @var Task $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($issueId);
            $validateCommentDescription = $this->validateCommentDescription($issue,$menuVersionId,$menuItemId);
            if($menuVersionId){
                $productVersion = InstanceProvider::getRepository()->getProductVersion()->findOneBy(array('id' => $menuVersionId));
                $issue->setProductVersion($productVersion);
            }else{
                $issue->setProductVersion(null);
            }

            /** @var \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion $productVersionDev */
            $productVersionDev = InstanceProvider::getService()->getProductVersion()->getProductVersion($versionDevId);
            $issue->setProductVersionDev($productVersionDev);

            $issue->setMenuItem($menuItemId);
            $this->entityManager->persist($issue);

            InstanceProvider::getService()->getCallLabels()->editCallRelationWithLabels($id, $callLabels);


            if(!empty($appointmentDate)){
                $productCall->setDtconpcuren($appointmentDate);
            }
            $this->entityManager->persist($productCall);
            $messageVersion = InstanceProvider::getRepository()->getComments()->getMessageVersion($issue->getId(),Messages::ISSUE_EDIT_VERSION,$product);
            $this->entityManager->flush();
            $connection->commit();

            /** @var Comments $fistCommnet */
            if($validateCommentDescription && $messageVersion && ($menuVersionId || $menuItemId)){
                $this->saveCommentOnIssuesByProductCallComment($issue->getId(), $messageVersion);
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function validateCommentDescription(Task $issue,$menuVersionId,$menuItemId){
        $validate = false;
        if($menuVersionId || $menuItemId){
            if($issue->getMenuItem() != $menuItemId){
                $validate = true;
            }else if($issue->getProductVersion() && $issue->getProductVersion()->getId() != $menuVersionId ){
                $validate = true;
            }else if(!$issue->getProductVersion() && !$menuVersionId){
                $validate = true;
            }
        }
        return $validate;
    }

    public function saveCommentOnIssuesByProductCallComment($issueId, $description) {
        $connection  = $this->entityManager->getConnection();
        $sendEmail = false;
        try {
            $this->entityManager->beginTransaction();
            /** @var Task $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($issueId);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=> CommentType::TYPE_COMMON));
            $comment = InstanceProvider::getFactory()->getComments()->createComments($issue, $description,$commentType);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();

            $connection->commit();
            $sendEmail = true;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        if($sendEmail){
            InstanceProvider::getService()->getComment()->sendsMailAdvisingAboutNewIssueComment($issueId, $description);
        }
    }


    private function updateRichClientTypeProductCall($id, $reference, $product, $productType, $productVersion, $useLicense,
                                                     $classification, $board, $description, $reportedBy, $callLabels,$cdCliente,
                                                     $appointmentDate,$productRelease,$productPatch)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            $productCall = $this->callRepository->find($id);
            $productCall = InstanceProvider::getFactory()->getCall()->updateCall($productCall, $reference, $product, $classification, $description);
            InstanceProvider::getService()->getCallLabels()->editCallRelationWithLabels($id, $callLabels);

            if(!empty($appointmentDate)){
                $productCall->setDtconpcuren($appointmentDate);
            }

            /** @var Entities\Soliserv $soliserv */
            $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $productCall->getCallRelationship()));
            $soliserv->setDsrefesoli($reference);
            $soliserv->setCdprodcomp($product);
            $soliserv->setCdprodgrup($productType);
            $soliserv->setCdverproorig($productVersion);
            $soliserv->setCdreleasorig($productRelease);
            $soliserv->setCdrelverorig($productRelease);
            $soliserv->setCdpatrelorig($productPatch);
            $soliserv->setCdassunto($classification);
            $soliserv->setCdoperador($reportedBy);
            $soliserv->setCdcliente($cdCliente);
            $soliserv->setNrlicencauso($useLicense);

            /** @var Entities\Descsoli $descsoli */
            $descsoli = InstanceProvider::getRepository()->getDescsoli()->findOneBy(array('nrsoliserv' => $productCall->getCallRelationship()));
            $descsoli->setDssoliserv($description);

            $this->entityManager->persist($descsoli);
            $this->entityManager->persist($soliserv);
            $this->entityManager->persist($productCall);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateSoliserv(Entities\CpCall $productCall, $reference, $product, $productType, $productVersion, $useLicense,
                                   $classification, $board, $description, $reportedBy, $callLabels,$cdCliente){
        /** @var Entities\Soliserv $soliserv */
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->find($productCall->getCallRelationship());
        $soliserv->setCdcliente($cdCliente);
        $soliserv->setDsrefesoli($description);
        $soliserv->setCdassunto($classification);

        $this->entityManager->persist($soliserv);
        $this->entityManager->flush();
    }

    /**
     * @param $callType
     * @param $id
     * @param $reference
     * @param $product
     * @param $classification
     * @param $description
     * @param $reportedBy
     * @param $callRelationship
     * @return int
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    private function validadeSaveCall($callType, $id, $reference, $product, $classification, $description, $reportedBy, $callRelationship)
    {
        if (empty($id)) {
            $productCall = InstanceProvider::getRepository()->getCpCall()->findOneBy(array("callRelationship" => $callRelationship, 'type' => $callType));
            if (!$productCall) {
                $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference, $reportedBy, $description, $callType,
                    $callRelationship, $product, $classification);

                $this->entityManager->persist($productCall);
                $this->entityManager->flush();
            }
            $id = $productCall->getNrcall();
        }
        return $id;
    }

    public function findCurrentUser(){
        $return = array(
            'CDOPERADOR' => Utilities::getEnvironment()->getCurrentCdoperador()
        );
        return $return;
    }

    /**
     * @param $param
     * @return string
     */
    public function checkAndCreateProductIdFilter($param)
    {
        if (!empty($param['search']) && in_array('PRODUCT_FILTER_ID', $param['search_in'])) {
            return "AND PDO.PROJECT_ID = {$param['search']}";
        } return '';
    }

    /**
     * @param array $filter
     * @param int $itemsPerPage
     * @param int $page
     * @return array
     */
    public function findProductCallProducts()
    {
        $projectRepository = InstanceProvider::getRepository()->getProject();
        $organization = $this->environment->getCurrentOrganizationId();
        if(!$this->environment->currentUserIsTeknisa()){
            $filterUserAcess = "AND PRD.PRODUCT_TYPE = 'C'";
        }
        else{
            $filterUserAcess = "";
        }
        $products = $projectRepository->findProducts($organization, 0, false, false, false, '', $filterUserAcess);
        $notModelProducts = DevOpsUtilities::removeModels($products, $organization);
        return $notModelProducts;
    }

    public function findProductCallProductsFromRichClient($productType, $params)
    {
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        return $this->getSistema($productType, '', '', '', $filterString);
    }

    private function getSistema($P_CDGRUPSIST = '', $P_CDSISTEMA = '', $P_NMSISTEMA = '', $P_TIPO = '', $filterString = '')
    {

        $adqSubProd = "
            SELECT
               DISTINCT(PR.CDARVPROD), PR.CDPRODUTO AS ID, PR.NMPRODUTO AS NAME
            FROM
               PARFILIALGEP PG, PRSUBGRPROD NV, PRODUTO PR,
               (SELECT CDARVPROD, CDPRODUTO FROM PRODUTO WHERE CDPRODUTO = NVL(:P_CDPRODGRUP,CDPRODUTO)) GRUP
            WHERE
                (PG.CDFILIAL = '9999')
                AND (NV.CDFILISUBGR = '9999' OR NV.CDFILISUBGR = 'P')
                AND (PG.CDGRUPPROD = NV.CDGRUPPROD)
                AND (NV.CDPRODUTO = PR.CDPRODUTO)
                AND (PR.CDARVPROD LIKE GRUP.CDARVPROD||'%')
                AND (PR.CDARVPROD = NVL(:P_CDARVPROD,PR.CDARVPROD))
                AND (INSTR (:P_CDPRODUTO, GRUP.CDPRODUTO) > 0)
        ";

        $adqSubProdParams = array(
            'P_CDPRODGRUP' => '',
            'P_CDARVPROD' => '',
            'P_CDPRODUTO' => ''
        );

        $adqProdGrupInt = "
            SELECT
               DISTINCT(PR.CDARVPROD), PR.CDPRODUTO AS ID, PR.NMPRODUTO AS NAME, PR.DSDIRBASEVER
            FROM
               PRODUTO PR,  NVSUBGRPROD NS, PARFILIALGEP PG
            WHERE
               (NS.CDGRUPPROD  = PG.CDGRUPPROD ) AND
               (NS.CDPRODUTO   = PR.CDPRODUTO  ) AND
               (PG.CDFILIAL    =    :P_CDFILIAL) AND
               (PR.CDARVPROD   = NVL(:P_CDARVPROD, PR.CDARVPROD)) AND
               (NS.CDFILISUBGR IN ('P', :P_CDFILIAL))
        ";

        $adqProdGrupIntParams = array(
            'P_CDFILIAL' => '',
            'P_CDARVPROD' => '',
        );

        $adqProdGrup = "
            SELECT CDPRODUTO
              FROM PRODUTO
             WHERE (SUBSTR(CDARVPROD,1,1) = 'P')
               AND (CDNVPRODUTO = :P_NVPRODUTO)
               AND (CDPRODUTO <> :P_CDPRODFIL)
        ";

        $adqProdGrupParams = array(
            'P_NVPRODUTO' => '',
            'P_CDPRODFIL' => ''
        );

        $stProdGrup = '';
        $cdProdGrup = '';
        try {
            if($P_TIPO == '') {
                if(!empty($P_CDSISTEMA) && !empty($P_NMSISTEMA)){
                    $adqSubProd = "SELECT * FROM ( " . $adqSubProd;
                    $adqSubProd .= " AND UPPER(PR." . $P_CDSISTEMA .") LIKE UPPER('" . $P_NMSISTEMA . '%' ."') ) WHERE 1 = 1 ". $filterString . "ORDER BY " . $P_CDSISTEMA;
                } else {
                    $adqSubProd = "SELECT * FROM ( " . $adqSubProd;
                    $adqSubProd .= ")\n WHERE 1 = 1" . $filterString;
                    $adqSubProd .= " ORDER BY CDARVPROD";
                }

                $cdGrupSist  = strtoupper($P_CDGRUPSIST);
                if($cdGrupSist != 'DOS' && $cdGrupSist != 'WEB' && $cdGrupSist != 'T') {
                    $stProdGrup = '_' . $P_CDGRUPSIST . '_';
                    $cdProdGrup = $P_CDGRUPSIST;
                } else {
                    $cdGrupSist  = strtoupper($P_CDGRUPSIST);

                    if ($cdGrupSist == 'DOS' ){
                        $stProdGrup = '_0000000102_';
                        $cdProdGrup = '0000000102';
                    }

                    if ($cdGrupSist == 'WEB' ){
                        $stProdGrup = '_0000000627_';
                        $cdProdGrup = '0000000627';
                    }

                    if ($cdGrupSist == 'T' ){
                        $cdProdGrup = '';
                        $stProdGrup = '';


                        /* adqProdGrup.ParamByName('P_NVPRODUTO').AsString := IntToStr(TTNiveis(ptProduto) - 1); */
                        $adqProdGrup['P_NVPRODUTO'] = '';
                        $adqProdGrup['P_CDPRODFIL'] = ' ';
                        $adqProdGrupResult = $this->entityManager->getConnection()->fetchAll($adqProdGrup, $adqProdGrupParams);

                        foreach($adqProdGrupResult as $adqProdGrupRow) {
                            $stProdGrup = $stProdGrup . '_' . $adqProdGrupRow['CDPRODUTO'];
                        }
                        $stProdGrup = $stProdGrup . '_';
                    }
                }

                $adqSubProdParams['P_CDPRODUTO'] = $stProdGrup;
                $adqSubProdParams['P_CDPRODGRUP'] = $cdProdGrup;
                $adqSubProdParams['P_CDARVPROD'] = '';

                $adqSubProdResults = $this->entityManager->getConnection()->fetchAll($adqSubProd, $adqSubProdParams);
                return $adqSubProdResults;
            } else {
                if(!empty($P_CDSISTEMA) && !empty($P_NMSISTEMA)){
                    $adqProdGrupInt = "SELECT * FROM ( " .  $adqProdGrupInt;
                    $adqProdGrupInt .= " AND UPPER(PR." . $P_CDSISTEMA . ") LIKE UPPER('" . $P_NMSISTEMA . '%' ."') ) WHERE 1 = 1" . $filterString . "ORDER BY " . $P_CDSISTEMA;
                } else {
                    $adqProdGrupInt = "SELECT * FROM ( " .  $adqProdGrupInt;
                    $adqProdGrupInt .= ")\n WHERE 1 = 1" . $filterString;
                    $adqProdGrupInt .= " ORDER BY PR.CDARVPROD";
                }

                $adqProdGrupInt['P_CDARVPROD'] = '';
                $adqProdGrupInt['P_CDFILIAL'] = '9999';

                $adqProdGrupIntResults = $this->entityManager->getConnection()->fetchAll($adqProdGrupInt, $adqProdGrupIntParams);
                return $adqProdGrupIntResults;
            }
        } catch(\Exception $e) {
            return array();
        }
    }

    /**
     * @param $params
     * @return mixed[]
     */
    public function findProductCallProductsFromTicket($params)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        return $cpCallRepository->findProductCallProductsFromTicket($filterString);
    }

    private function getFilterVersionNotLTS($resourceId, $projectId)
    {
        $nrOrg = InstanceProvider::getRepository()->getResource()->findOrganizationByResourceId($resourceId);
        $productProject = $this->getProductProjectByProjectId($projectId);
        if($productProject && !empty($nrOrg)){
            $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $productProject->getId()));
            $omProduto = InstanceProvider::getRepository()->getOmProduto()->findOneBy(array('nrexternalProductId' => $product->getId()));
            $nrOrgId = $nrOrg['NRORG'];
            if ($omProduto) {
                $nrprodutoId = $omProduto->getId();
                $filterNotLTS = "SELECT OV.PRODUCT_VERSION_ID 
                                   FROM OM_PRODUTO_ORG OP
                                   JOIN OM_PRODUTO_ORG_VERSAO OV ON OV.NRPRODUTO_ORG_ID = OP.ID
                                  WHERE OP.NRORG_ID = $nrOrgId
                                    AND OP.NRPRODUTO_ID = $nrprodutoId";
                return $filterNotLTS;
            }
            return 0;
        }
        return 0;
    }

    private function getProductProjectByProjectId($projectId)
    {
        /** @var Entities\Project $project */
        $project = InstanceProvider::getRepository()->getProject()->find($projectId);
        if($project){
            return InstanceProvider::getService()->getProject()->getProductProject($project);
        }
        return null;
    }
}
